//+------------------------------------------------------------------+
//|                                                 SampleSignal.mqh |
//|                        Copyright 2010, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2010, MetaQuotes Software Corp."
#property link      "http://www.mql5.com"
//+------------------------------------------------------------------+
//|                                                   |
//+------------------------------------------------------------------+
#include <Expert\ExpertSignal.mqh>
#include <Indicators\BillWilliams.mqh>
// wizard description start
//+------------------------------------------------------------------+
//| Description of the class                                         |
//| Title=                                          |
//| Type=SignalAdvanced                                              |
//| Name=IFA                                                         |
//| Class=CSampleSignal                                              |
//| Page=                                                            |
//| Parameter=Limit,double,0.0                                       |
//| Parameter=StopLoss,double,50.0                                   |
//| Parameter=TakeProfit,double,50.0                                 |
//| Parameter=Expiration,int,10                                      |
//+------------------------------------------------------------------+
// wizard description end
//+------------------------------------------------------------------+
//|  CSampleSignal.                                             |
//| :          |
//|                                            |
//|                 .                     |
//|                 CExpertSignal.        |
//+------------------------------------------------------------------+
class CSampleSignal : public CExpertSignal
  {
protected:

   CiFractals         m_fractal;                      //  
   CiFractals         m_FUP;                      //  
   CiFractals         m_FDN;                      //  
   CiOpen             m_open;                    //       
   CiClose            m_close;                   //       
   //---  
      
   ENUM_MA_METHOD     m_method_ma;               //    
   ENUM_APPLIED_PRICE m_applied_ma;              //    
   double             m_limit;                   //       
   double             m_stop_loss;               //    "stop loss"   
   double             m_take_profit;             //    "take profit"   
   int                m_expiration;              //  ""    
   double               m_size;                       //  
   int                m_pattern_0;                 //  
   int                m_pattern_1;                 //  
   public:
                      CSampleSignal();
   //---    

   void               AppliedMA(ENUM_APPLIED_PRICE value) { m_applied_ma=value;             }
   void               Limit(double value)                 { m_limit=value;                  }
   void               StopLoss(double value)              { m_stop_loss=value;              }
   void               TakeProfit(double value)            { m_take_profit=value;            }
   void               Expiration(int value)               { m_expiration=value;             }
   void               Size(double value)                  { m_size=value;                   }
   
   //---    
   virtual bool       ValidationSettings();
   //---    
   virtual bool       InitIndicators(CIndicators* indicators);
   //---      
   //   virtual int       CheckOpenLong(double& price,double& sl,double& tp,datetime& expiration);
   //   virtual int       CheckOpenShort(double& price,double& sl,double& tp,datetime& expiration);
   
  //---      
   virtual int       CheckOpenLong();
   virtual int       CheckOpenShort();

protected:
   //---   
   bool               InitF(CIndicators* indicators);
   bool               InitOpen(CIndicators* indicators);
   bool               InitClose(CIndicators* indicators);
   //---     
   double             FUP(int index)                      { return(m_fractal.Upper(index));       }
   double             FDN(int index)                      { return (m_fractal.Lower(index));       }
   double             Open(int index)                     { return(m_open.GetData(index));  }
   double             Close(int index)                    { return(m_close.GetData(index)); }
  };
//+------------------------------------------------------------------+
//|  CSampleSignal.                                       |
//| INPUT:  .                                                     |
//| OUTPUT: .                                                     |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
void CSampleSignal::CSampleSignal()
  {
//---   -
  
   m_limit      =10.0;
   m_stop_loss  =50.0;
   m_take_profit=50.0;
   m_expiration =10;
   m_pattern_0=50;
   m_pattern_1=50;
   Print("void CSampleSignal::CSampleSignal()");
  }
//+------------------------------------------------------------------+
//|   .                                   |
//| INPUT:  .                                                     |
//| OUTPUT: true-  ,  false.             |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool CSampleSignal::ValidationSettings()
  {
  
  Print("CSampleSignal::ValidationSettings()");
  if (!CExpertSignal::ValidationSettings())
  {
  printf(__FUNCTION__+": ");
  return ( false );
   }
//---  
   
//---  
   return(true);
  }
//+------------------------------------------------------------------+
//|    .                           |
//| INPUT:  indicators -   -               |
//|                        .                    |
//| OUTPUT: true-   ,  false.         |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool CSampleSignal::InitIndicators(CIndicators* indicators)
  {
  
  Print("CSampleSignal::InitIndicators(CIndicators* indicators)");
//---  
   if(indicators==NULL)       return(false);
//---   
   if(!InitF(indicators))    return(false);
//---    
   if(!InitOpen(indicators))  return(false);
//---    
   if(!InitClose(indicators)) return(false);
//---  
   return(true);
  }
//+------------------------------------------------------------------+
//|   .                                |
//| INPUT:  indicators -   -               |
//|                        .                    |
//| OUTPUT: true-   ,  false.         |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool CSampleSignal::InitF(CIndicators* indicators)
  {
  
  Print("CSampleSignal::InitMA(CIndicators* indicators)");  
  //---    
  if(!m_fractal.Create(m_symbol.Name(),m_period))
     {
      printf(__FUNCTION__+":   ");
      return(false);
     }

   //---    
   if(!indicators.Add(GetPointer(m_fractal)))
     {
      printf(__FUNCTION__+":   ");
      return(false);
     }
     
   //m_size=BarsCalculated(m_cif.Handle); 
   //printf("m_size", NormalizeDouble(m_size, 8)); 
     
   //---  
   return(true);
  }
//+------------------------------------------------------------------+
//|    .                            |
//| INPUT:  indicators -   -               |
//|                        .                    |
//| OUTPUT: true-   ,  false.         |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool CSampleSignal::InitOpen(CIndicators* indicators)
  {
  
   Print("CSampleSignal::InitOpen(CIndicators* indicators)");  
//---   
   if(!m_open.Create(m_symbol.Name(),m_period))
     {
      printf(__FUNCTION__+":   ");
      return(false);
     }
//---    
   if(!indicators.Add(GetPointer(m_open)))
     {
      printf(__FUNCTION__+":   ");
      return(false);
     }
//---  
   return(true);
  }
//+------------------------------------------------------------------+
//|    .                            |
//| INPUT:  indicators -   -               |
//|                        .                    |
//| OUTPUT: true-   ,  false.         |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool CSampleSignal::InitClose(CIndicators* indicators)
  {
  
     Print("CSampleSignal::InitClose(CIndicators* indicators)"); 
  
//---   
   if(!m_close.Create(m_symbol.Name(),m_period))
     {
      printf(__FUNCTION__+":   ");
      return(false);
     }
//---    
   if(!indicators.Add(GetPointer(m_close)))
     {
      printf(__FUNCTION__+":   ");
      return(false);
     }
//---  
   return(true);
  }
//+------------------------------------------------------------------+
//|     .                         |
//+------------------------------------------------------------------+
//bool CSampleSignal::CheckOpenLong(double& price,double& sl,double& tp,datetime& expiration)
int CSampleSignal::CheckOpenLong()
  {
  Print("CSampleSignal::CheckOpenLong"); // 
  int n;
  double unit=PriceLevelUnit();
  int result=0;
    //---     for     
   for(n=0; n<1000; n++)
     {
     Print("n  ",  n);
     //---   ,  
      if(m_fractal.Upper(n)!=EMPTY_VALUE)
         {
         Print("m_fractal.Upper",  m_fractal.Upper(n));
         break;
         }
     }
     
   //---       
   double   UpFractal_1=NormalizeDouble(m_fractal.Upper(n),8);
   //---      
  int   UpperFractal=n;
  Print("double   UpFractal_1", UpFractal_1);
  //         
  double  x=NormalizeDouble(m_close.GetData(1),8);
  Print("double x", x);
  if (x>UpFractal_1)
  {
      double price=m_symbol.NormalizePrice(m_symbol.Ask()+m_limit*unit);
      double sl   =m_symbol.NormalizePrice(price-m_stop_loss*unit);
      double tp   =m_symbol.NormalizePrice(price+m_take_profit*unit);
      datetime expiration=m_expiration*PeriodSeconds(m_period);
      
      result=m_pattern_0;
      return(result);
  }
  
return(result);
  }
//+------------------------------------------------------------------+
//|     .                         |
//+------------------------------------------------------------------+
//bool CSampleSignal::CheckOpenShort(double& price,double& sl,double& tp,datetime& expiration)
int CSampleSignal::CheckOpenShort()
  {
  Print("CSampleSignal::CheckOpenShort"); // 
  double unit=PriceLevelUnit();
  int n=0;
  int result=0;
  
  //---    
   //---    
   for(n=0; n<1000; n++)
     {
//---   ,  
      if(m_fractal.Lower(n)!=EMPTY_VALUE)
         {
         break;
         }
      }
//---       
   double  LowFractal_1=NormalizeDouble(m_fractal.Lower(n),8);
//---      
   int   LowerFractal=n;
   Print("LowFractal_1",  LowFractal_1);
   //         
   double  x=NormalizeDouble(m_close.GetData(1),8);
    
    Print ("double  x", x);
  if (x<LowFractal_1)
  {
  
      double price=m_symbol.NormalizePrice(m_symbol.Bid()+m_limit*unit);
      double sl   =m_symbol.NormalizePrice(price+m_stop_loss*unit);
      double tp   =m_symbol.NormalizePrice(price-m_take_profit*unit);
      datetime expiration=m_expiration*PeriodSeconds(m_period);
      
      result=m_pattern_1;
      return(result);
  }

//---   
   return(result);
  }

//+------------------------------------------------------------------+
